# Copyright (C) 2007 by Intevation GmbH
# Authors:
# Bernhard Herzog <bh@intevation.de>
#
# This program is free software under the GPL (>=v2)
# Read the file COPYING coming with the software for details.

"""Tests for ThreadPool"""

import threading
import unittest

import inteproxy.threadpool

def currentThreadSet():
    """Returns the currently running threads as a set"""
    return set(threading.enumerate())


class TestThreadPool(unittest.TestCase):

    def test_thread_pool(self):

        # handler function for the thread pool that simply records the
        # items it processes.
        processed_items = []
        def handler(item):
            processed_items.append(item)

        # snapshot of currently running threads so that we can keep
        # track of new threads.
        initial_threads = currentThreadSet()

        # Create a thread pool and start its threads
        pool = inteproxy.threadpool.ThreadPool(5, handler)
        pool.start()

        # check that 5 new threads have been started
        new_threads = currentThreadSet() - initial_threads
        self.assertEquals(len(new_threads), 5)

        # put some things into the queue.  We cannot check immediately
        # whether they've been processed, because that happens in a
        # different thread.  processed
        pool.put("abc")
        pool.put("def")

        # stop the threads and check that they're gone
        pool.stop()
        remaining_threads = currentThreadSet() - new_threads - initial_threads
        self.assertEquals(len(remaining_threads), 0)

        # check whether all items put into the queue have been
        # processed.
        self.assertEquals(sorted(processed_items), ["abc", "def"])
